function output=Curvature(x,y)

%This function computes a finite-difference approximation of the local
%radius of curvature R of a curve (x(t),y(t)) and of the associated centers
%of curvature O(t) in Cartesian coordinates . The inputs are the
%coordinates, both of length N, (x,y) of a continuous curve L (line
%vectors). The output is a matrix whose first line is the local radius of
%curvature R of L, the second and third lines being the coordinates (Ox,Oy)
%of the local centers of curvature. The size of the output is (3,N), the
%first and  last values in a row being set as NaN.
%
%Use: CurvOutput=Curvature(x,y);
%     R=CurvOutput(1,:);
%     Ox=CurvOutput(2,:);
%     Oy=CurvOutput(3,:);
%
% (c) Aymeric-Pierre B. Peyret 2008-2011

%Preallocation of space for data
xp=zeros(size(x));xs=zeros(size(x));
yp=zeros(size(y));ys=zeros(size(y));
C=zeros(size(x));

%BIG assumption, satisfied only if intervals between a datapoint and both
%its neighbors are all the same.
dt=1;

%Computation of finite-difference estimates of t-derivates
xp(1)=NaN;yp(1)=NaN;xp(length(x))=NaN;yp(length(y))=NaN;
xs(1)=NaN;ys(1)=NaN;xs(length(x))=NaN;ys(length(y))=NaN;
C(1)=NaN;C(length(C))=NaN;
for t=2:(length(x)-1)
    xp(t)=(x(t+dt)-x(t-dt))/(2*dt);
    yp(t)=(y(t+dt)-y(t-dt))/(2*dt);
    xs(t)=(x(t+dt)+x(t-dt)-2*x(t))/(dt^2);
    ys(t)=(y(t+dt)+y(t-dt)-2*y(t))/(dt^2);
end

%Computing curvatures
C=(xp.*ys-xs.*yp)./((xp.^2+yp.^2).^(3/2));

%Computing radii of curvature, inverse of curvature
R=1./C;

%Computing normalization factors of (x,y)-normal vector
modmp=((xp.^2+yp.^2).^(1/2));

%Computing coordinates of centers of curvature
Ox=x-R./modmp.*yp;
Oy=y+R./modmp.*xp;

%Creating output variables
Omega=[Ox;Oy];
output=[R;Omega];