function [ChanWidth,varargout]=Width(x,y,xbi1,ybi1,xbi2,ybi2,windowsize,theta,useold,SwipeLength,filteringoff)

%This function computes the width of a channel, and calls the desired
%curvature function to also output radius of curvature, coordinates of the
%centers of curvature (evolute) of a given curve (x,y)
%
%To run with all inputs and outputs, use the following syntax:
%[ChanWidth,ChanWidth2,R,coord1,coord2,width1,width2,Ox,Oy,xdir,ydir,coordfilt,bankfilt1,bankfilt2,mindist1,mindist2,maxival1,maxival2]=Width(x,y,xbi1,ybi1,xbi2,ybi2,windowsize,theta,useold,SwipeLength,filteringoff);
%
%To run the code in "curvature-only mode" (no width computation, just
%filtering and curvature), use the syntax:
%[R,Ox,Oy,coordfilt]=Width(x,y,NaN,NaN,NaN,NaN,windowsize,theta,useold,NaN,filteringoff);
%
%Outputs:
% - R is the radius of curvature of the filtered curve whose unfiltered
%(raw) data were (x,y)
% - Ox and Oy are the coordinates of the associated centers of curvature
% - coordfilt are the coordinates of the filtered (x,y) curve (2-column
% vector)
% - ChanWidth is the width of the channel, sum of two half-widths going
% through the filtered centerline points
% - ChanWidth2 is the width of the channel without going through centerline
% points
% - width1 and width2 are the half-widths, related to bank1 and bank2
% - bankfilt1 and bankfilt2 are the filtered bank coordinates (2x 2-column
% vectors)
% - mindist1, mindist2, maxival1, maxival2 are debugging parameters
% - Only the first output argument has to be specified, others are
% compulsory. If used, however, they need to be added in the same order as
% described above.
%
%Inputs:
% - x: abscissae of the centerline ("full" width mode) or abscissae of the
% curve whose radii and centers of curvature need to be determined after
% smoothing ("curvature-only" mode)
% - y: ordinates of the centerline ("full" width mode) or ordinates of the
% curve whose radii and centers of curvature need to be determined after
% smoothing ("curvature-only" mode)
% - xbi1: abscissae of bank 1 ("full" width mode) or NaN ("curvature-only"
% mode)
% - ybi1: ordinates of bank 1 ("full" width mode) or NaN ("curvature-only"
% mode)
% - xbi2: abscissae of bank 2 ("full" width mode) or NaN ("curvature-only"
% mode)
% - ybi2: ordinates of bank 2 ("full" width mode) or NaN ("curvature-only"
% mode)
% - windowsize: Length of composite smoothing window. Needs to be 1+a
% multiple of 4, so that the length of the Ricker window can be an odd
% number and ~half of the value of windowsize. Default value is 21. Larger
% values decrease the scattering due to human digitization but also
% slightly change the shape of the channel
% - theta: fraction of Ricker window used for smoothing. (1-theta) is the
% fraction of the Chebyshev window used for filtering. Default value is 1.
% - useold: useold=1 implies use of (x(t),y(t)) basic curvature code, while
% useold=0 implies use of (x,y(x)) basic curvature code. useold=0 is
% strongly advised for natural channels and is the default value.
% - SwipeLength: number of points around a central point that are used to
% look for intersections between the bank and the normal to the centerline.
% Default value is 11, i.e. 5 points on each side of the central point. May
% be set as NaN when function used in "curvature-only" mode.
% - filteringoff: filteringoff=1 bypasses the filtering process so that
% only raw data (unfiltered user inputs) are used. By default, filtering is
% used so filteringoff=0 is the default value
%
%First six input variables are compulsory; other input variables are
%optional; when omitted, default values are used.
%
%(c) Aymeric-Pierre B. Peyret 2010-2011


%initial time
initime=tic;

%I. - PRELIMINARIES: make sure variables are what they should be...

%I.1 - Adds default values when user did not input them
if nargin<10
    filteringoff=0;
end
if nargin<9
    SwipeLength=11;
elseif nargin<8
    useold=0;
elseif nargin<7
    theta=1;
elseif nargin<6
    windowsize=21;
end

%I.2 - Checks that x and y, xbi1 and ybi1, xbi2 and ybi2 are column vectors and are the same size

sx=size(x);sy=size(y);
sx1=size(xbi1);sy1=size(ybi1);
sx2=size(xbi2);sy2=size(ybi2);

if sx(1)~=sy(1)
    if sx(1)~=sy(2)
        error('Sizes...','The sizes of vectors x and y are different. Please correct!');
    elseif sy(1)~=sx(2)
        error('Sizes...','The sizes of vectors x and y are different. Please correct!');
    else
        y=y';
        sy=size(y);
    end
else
    if sx(2)~=sy(2)
        error('Sizes...','The sizes of vectors x and y are different. Please correct!');
    end
end

if sx1(1)~=sy1(1)
    if sx1(1)~=sy1(2)
        error('Sizes...','The sizes of vectors xbi1 and ybi1 are different. Please correct!');
    elseif sy1(1)~=sx1(2)
        error('Sizes...','The sizes of vectors xbi1 and ybi1 are different. Please correct!');
    else
        ybi1=ybi1';
        sy1=size(ybi1);
    end
else
    if sx1(2)~=sy1(2)
        error('Sizes...','The sizes of vectors xbi1 and ybi1 are different. Please correct!');
    end
end

if sx2(1)~=sy2(1)
    if sx2(1)~=sy2(2)
        error('Sizes...','The sizes of vectors xbi2 and ybi2 are different. Please correct!');
    elseif sy2(1)~=sx2(2)
        error('Sizes...','The sizes of vectors xbi2 and ybi2 are different. Please correct!');
    else
        ybi2=ybi2';
        sy2=size(ybi2);
    end
else
    if sx2(2)~=sy2(2)
        error('Sizes...','The sizes of vectors xbi2 and ybi2 are different. Please correct!');
    end
end

if (sx(1)==1)&&(sx(2)~=1)
    x=x';y=y';%make them column vectors
end

if (sx1(1)==1)&&(sx1(2)~=1)
    xbi1=xbi1';ybi1=ybi1';%make them column vectors
end

if (sx2(1)==1)&&(sx2(2)~=1)
    xbi2=xbi2';ybi2=ybi2';%make them column vectors
end

%I.3 - Find if code is run in "curvature only" mode, i.e. if at least one
%argument xbi1,ybi1,xbi2,ybi2 only has NaN values

curvatureonly=1-(length(find(~isnan(xbi1)))*length(find(~isnan(ybi1)))*length(find(~isnan(xbi2)))*length(find(~isnan(ybi2)))>0);

%I.4 - Checks that the smoothing window length=4*X+1, X integer, and the
%swipe length is an odd number.

if mod(windowsize-1,4)~=0
    while mod(windowsize-1,4)~=0
        windowsize=windowsize+1;
    end
    display(['Window size was not a multiple a 4+1, and was increased to' num2str(windowsize)])
end

if ~curvatureonly
    if mod(SwipeLength-1,2)
        errordlg('Please choose an odd swipelength');
        SwipeLength=SwipeLength+1;
    end
end

%II. PRETREATMENT: FILTERING AND RADIUS OF CURVATURE

%II.1 - Filtering

if filteringoff%Bypass filtering: filtered <- unfiltered
    xfilt=x;yfilt=y;
    xb1=xbi1;yb1=ybi1;
    xb2=xbi2;yb2=ybi2;
else%apply same filter on all datasets: centerline, bank1, bank2
    [xfilt,yfilt]=Filtering(x,y,windowsize,theta);
    if ~curvatureonly
        %[xb1,yb1]=Filtering(xbi1,ybi1,windowsize,theta);
        %[xb2,yb2]=Filtering(xbi2,ybi2,windowsize,theta);
        xb1=xbi1;yb1=ybi1;
        xb2=xbi2;yb2=ybi2;
    end
end

%II.1 - Computation of radius of curvature

if useold%use (x(t),y(t)) curvature code, called "old curvature code"
    Courb2=Curvature(xfilt',yfilt');
    R=Courb2(1,:)';
    Ox=Courb2(2,:)';
    Oy=Courb2(3,:)';
    clear Courb2;
else%use (x,y(x)) curvature code, called "new curvature code"
    [R,Ox,Oy]=Courbure(xfilt,yfilt);
%     [R,Ox,Oy,horizontal,vertical,dxzero]=CourbureOld(xfilt,yfilt);
    %     Ox=Ox';
    %     Oy=Oy';
    % R=R';
end

%II.2 - If code in "curvature only mode", bypass width computations and
%only create outputs

if curvatureonly
    nout = max(nargout,1)-1;
    ChanWidth=R;
    if nout>0
        varargout(1)={Ox};
    end
    if nout>1
        varargout(2)={Oy};
    end
    if nout>2
        coordfilt=[xfilt yfilt];
        varargout(3)={coordfilt};
    end
    endtime=toc(initime);
else
    
    %III. WIDTH CODE, First part: normal vectors, minimum-distance points
    
    %III.1 - Computation of normalized centerline-normal vectors
    
    VecOx=Ox-xfilt;%x-component of centerline-normal vector
    VecOy=Oy-yfilt;%y-component of centerline-normal vector
    NormVecOMx=VecOx./sqrt(VecOx.^2+VecOy.^2);%x-component of centerline-normal vector, normalized
    NormVecOMy=VecOy./sqrt(VecOx.^2+VecOy.^2);%y-component of centerline-normal vector, normalized
    
    %directions of the centerline-normal vector
    xdir1=NormVecOMx.*((R<=0).*(1)+(R>0).*(-1));%x-component of centerline-normal vector
    ydir1=NormVecOMy.*((R<=0).*(1)+(R>0).*(-1));%y-component of centerline-normal vector
    
    %opposite centerline-normal vector
    xdir2=NormVecOMx.*((R<=0).*(-1)+(R>0).*(1));
    ydir2=NormVecOMy.*((R<=0).*(-1)+(R>0).*(1));
    
%     if ~useold%when using new curvature code, add the horizontal and double-vertical centerline-normal vectors
%         VecOx(horizontal)=zeros(size(VecOx(horizontal)));
%         VecOy(horizontal)=ones(size(VecOy(horizontal)));
%         NormVecOMx(horizontal)=zeros(size(NormVecOMx(horizontal)));
%         NormVecOMy(horizontal)=ones(size(NormVecOMy(horizontal)));
%         xdir1(horizontal)=zeros(size(xdir1(horizontal)));
%         ydir1(horizontal)=-ones(size(ydir1(horizontal)));
%         xdir2(horizontal)=zeros(size(xdir2(horizontal)));
%         ydir2(horizontal)=ones(size(ydir2(horizontal)));
%         
%         VecOy(vertical)=zeros(size(VecOx(vertical)));
%         VecOx(vertical)=ones(size(VecOy(vertical)));
%         NormVecOMy(vertical)=zeros(size(NormVecOMx(vertical)));
%         NormVecOMx(vertical)=ones(size(NormVecOMy(vertical)));
%         ydir1(vertical)=zeros(size(xdir1(vertical)));
%         xdir1(vertical)=-ones(size(ydir1(vertical)));
%         ydir2(vertical)=zeros(size(xdir2(vertical)));
%         xdir2(vertical)=ones(size(ydir2(vertical)));
%     end
    
    %III.1 - Computation of minimum-distance point indices
    
    SL2=(SwipeLength-1)/2;%Number of points on each side of minimum-distance point
    
    %Compute distance matrices between each centerline-normal point and each
    %bank point:
    % for bank one
    distmat1=sqrt((ones(length(xfilt),1)*(xb1')-(xfilt*ones(1,length(xb1)))).^2+(ones(length(yfilt),1)*(yb1')-(yfilt*ones(1,length(yb1)))).^2);
    % for bank two
    distmat2=sqrt((ones(length(xfilt),1)*(xb2')-(xfilt*ones(1,length(xb2)))).^2+(ones(length(yfilt),1)*(yb2')-(yfilt*ones(1,length(yb2)))).^2);
    
    mindist1=zeros(size(xfilt));
    mindist2=zeros(size(xfilt));
    
    for i=1:length(xfilt)%mindist1/2 contain the minimum-distance points
        mindist1(i)=find(distmat1(i,:)==min(distmat1(i,:)));
        mindist2(i)=find(distmat2(i,:)==min(distmat2(i,:)));
    end
    
    %III.2 - Computation of PM vector coordinates, P centerline points, M
    %points around minimum-distance points
    
    matvecx1=xfilt*ones(1,SwipeLength);
    matvecy1=yfilt*ones(1,SwipeLength);
    matvecx2=xfilt*ones(1,SwipeLength);
    matvecy2=yfilt*ones(1,SwipeLength);
    % normfact1=yfilt*ones(1,SwipeLength);
    % normfact2=yfilt*ones(1,SwipeLength);
    % prodscal1=xfilt*ones(1,SwipeLength);
    % prodscal2=xfilt*ones(1,SwipeLength);
    
    for j=1:length(xfilt)
        %     [j j-SL2 j+SL2]
        xtop1=[zeros(1,SL2-mindist1(j)+1)*NaN xb1(max(mindist1(j)-SL2,1):min(mindist1(j)+SL2,length(xb1)))' zeros(1,mindist1(j)+SL2-length(xb1))*NaN];
        ytop1=[zeros(1,SL2-mindist1(j)+1)*NaN yb1(max(mindist1(j)-SL2,1):min(mindist1(j)+SL2,length(yb1)))' zeros(1,mindist1(j)+SL2-length(yb1))*NaN];
        xtop2=[zeros(1,SL2-mindist2(j)+1)*NaN xb2(max(mindist2(j)-SL2,1):min(mindist2(j)+SL2,length(xb2)))' zeros(1,mindist2(j)+SL2-length(xb2))*NaN];
        ytop2=[zeros(1,SL2-mindist2(j)+1)*NaN yb2(max(mindist2(j)-SL2,1):min(mindist2(j)+SL2,length(yb2)))' zeros(1,mindist2(j)+SL2-length(yb2))*NaN];
        matvecx1(j,:)=xtop1-matvecx1(j,:);
        matvecy1(j,:)=ytop1-matvecy1(j,:);
        matvecx2(j,:)=xtop2-matvecx2(j,:);
        matvecy2(j,:)=ytop2-matvecy2(j,:);
    end
    normfact1=sqrt(matvecx1.^2+matvecy1.^2);
    normfact2=sqrt(matvecx2.^2+matvecy2.^2);
    
    %Normalized PM vectors
    matvecx1=matvecx1./normfact1;
    matvecy1=matvecy1./normfact1;
    matvecx2=matvecx2./normfact2;
    matvecy2=matvecy2./normfact2;
    
    %III.3 - Computation of dot products between PM and centerline-normal
    %or centerline-tangent vectors
    
    %Scalar product PM.centerline-normal vector
    prodscal1=(xdir1*ones(1,SwipeLength)).*matvecx1+(ydir1*ones(1,SwipeLength)).*matvecy1;
    prodscal2=(xdir1*ones(1,SwipeLength)).*matvecx2+(ydir1*ones(1,SwipeLength)).*matvecy2;
    
    %Scaler product PM.centerline-normal vector
    projscal1=-(ydir1*ones(1,SwipeLength)).*matvecx1+(xdir1*ones(1,SwipeLength)).*matvecy1;
    projscal2=-(ydir1*ones(1,SwipeLength)).*matvecx2+(xdir1*ones(1,SwipeLength)).*matvecy2;
    
    %III.4 - Pre-allocationf of width lists and other computation-helping
    %lists..
    
    width1=zeros(length(xfilt),1);
    width2=zeros(length(xfilt),1);
    
    maxival1=zeros(length(xfilt),1);
    maxival2=zeros(length(xfilt),1);
    
    coord1=zeros(length(xfilt),2);
    coord2=zeros(length(xfilt),2);
    
    %III.5 - First pass of width determinations
    
    for j=1:length(xfilt)
        % maxi1/2: in case there is a bank1/bank2 point right on the
        % centerline-normal, finds its(/their) index. Very rarely happens with
        % real data, but always worth the try...
        maxi1=find(abs(round(prodscal1(j,:)*10^4))==10^4);
        maxi2=find(abs(round(prodscal2(j,:)*10^4))==10^4);
        %III.5.a - subpart for bank 1
        if length(maxi1)>=1
            if length(maxi1)~=1
                maximaxi1= 1./(abs(1-abs(prodscal1(j,maxi1))))==max(1./abs(1-abs(prodscal1(j,maxi1))));
                maxi1=maxi1(maximaxi1);
                %             maximaxi1=find(1./(abs(1-abs(prodscal1(j,maxi1))))==max(1./abs(1-abs(prodscal1(j,maxi1)))));
                %             maxi1=maxi1(maximaxi1);
            end
            width1(j)=sqrt((xb1(mindist1(j)-(SL2+1)+maxi1)-xfilt(j)).^2+(yb1(mindist1(j)-(SL2+1)+maxi1)-yfilt(j)).^2);
            maxival1(j)=maxi1;
            coord1(j,:)=[xb1(mindist1(j)-(SL2+1)+maxi1) yb1(mindist1(j)-(SL2+1)+maxi1)];
        else
            %When no point (bank1) right on the centerline-normal is found,
            %then look for the point PL that makes the largest (in absolute
            %value...) dot-product (Centerline-PL).(centerline-normal)
            maxi1=find(1./(abs(1-abs(prodscal1(j,:))))==max(1./abs(1-abs(prodscal1(j,:)))));%use function 1/|1-|x|| for points closest to -1 and 1
            cosa=abs(prodscal1(j,maxi1));
            if length(maxi1)==1
                if (maxi1~=1).*(maxi1~=SwipeLength)%Check that this PL point is NOT an extremity of the swipe vector
                    if projscal1(j,maxi1-1)*projscal1(j,maxi1)<0%Check if point PLL on the left of PL has opposite sign (PLL-Centerline).(centerline-tangent) projection
                        cosb=abs(prodscal1(j,maxi1-1));
                        [width1(j),coord1(j,1),coord1(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb1(mindist1(j)-(SL2+1)+maxi1),yb1(mindist1(j)-(SL2+1)+maxi1),xb1(mindist1(j)-(SL2+1)+maxi1-1),yb1(mindist1(j)-(SL2+1)+maxi1-1),xdir1(j),ydir1(j));
                    elseif projscal1(j,maxi1+1)*projscal1(j,maxi1)<0%Check if point PLR on the right of PL has opposite sign (PLR-Centerline).(centerline-tangent) projection
                        cosb=abs(prodscal1(j,maxi1+1));
                        [width1(j),coord1(j,1),coord1(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb1(mindist1(j)-(SL2+1)+maxi1),yb1(mindist1(j)-(SL2+1)+maxi1),xb1(mindist1(j)-(SL2+1)+maxi1+1),yb1(mindist1(j)-(SL2+1)+maxi1+1),xdir1(j),ydir1(j));
                    else %wrong position of maxi index
                        maxi1=find(projscal1(j,1:SwipeLength-1).*projscal1(j,2:SwipeLength)<0);%look for two points within swipe length with opposite (Centerline-point) projections on the centerline tangent
                        if ~isempty(maxi1)
                            if length(maxi1)>1
                                maximaxi1=find(1-abs(prodscal1(j,maxi1))==min(1-abs(prodscal1(j,maxi1))));
                                maxi1=maxi1(maximaxi1);
                            end
                            cosa=abs(prodscal1(j,maxi1));cosb=abs(prodscal1(j,maxi1+1));
                            [width1(j),coord1(j,1),coord1(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb1(mindist1(j)-(SL2+1)+maxi1),yb1(mindist1(j)-(SL2+1)+maxi1),xb1(mindist1(j)-(SL2+1)+maxi1+1),yb1(mindist1(j)-(SL2+1)+maxi1+1),xdir1(j),ydir1(j));
                        else%cannot find any point within swipe length close enough to the normal
                            width1(j)=NaN;
                            coord1(j,:)=[NaN NaN];
                        end
                    end
                    if ~isempty(maxi1), maxival1(j)=maxi1; else maxival1(j)=NaN; end
                else%if PL is an extremity of the swipe vector; usually happens at beginning and end of datasets
                    maxi1temp=maxi1;
                    maxi1=find(projscal1(j,1:SwipeLength-1).*projscal1(j,2:SwipeLength)<0);
                    if ~isempty(maxi1)%wrong position of maxi index
                        if length(maxi1)>1%found other position of point within swipe length such that a neighboring point has an opposite projection on centerline-tangent
                            maximaxi1=find(1-abs(prodscal1(j,maxi1))==min(1-abs(prodscal1(j,maxi1))));
                            maxi1=maxi1(maximaxi1);
                        end
                        cosa=abs(prodscal1(j,maxi1));cosb=abs(prodscal1(j,maxi1+1));
                        [width1(j),coord1(j,1),coord1(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb1(mindist1(j)-(SL2+1)+maxi1),yb1(mindist1(j)-(SL2+1)+maxi1),xb1(mindist1(j)-(SL2+1)+maxi1+1),yb1(mindist1(j)-(SL2+1)+maxi1+1),xdir1(j),ydir1(j));
                        %                 elseif prodscal1(j,:)<cosd(80)%point accepted if angle with centerline-tangent more than 80degrees
                        %                     maxi1=maxi1temp;
                        %                     width1(j)=sqrt((xb1(mindist1(j)-(SL2+1)+maxi1)-xfilt(j)).^2+(yb1(mindist1(j)-(SL2+1)+maxi1)-yfilt(j)).^2);
                        %                     coord1(j,:)=[xb1(mindist1(j)-(SL2+1)+maxi1) yb1(mindist1(j)-(SL2+1)+maxi1)];
                    else%cannot find any point within swipe length close enough to the normal
                        width1(j)=NaN;
                        coord1(j,:)=[NaN NaN];
                    end
                end
            else%cannot find any point that maximizes the dot product between the centerline-normal vector and the centerline-point vector. Usually means that every value is NaN...
                width1(j)=NaN;
                coord1(j,:)=[NaN NaN];
            end
        end
        %III.5.b - subpart for bank 2 - This part is the same as part III.5.a
        %except that indices "1" have been changed to indices "2"... Code lines
        %will therefore NOT have any comment associated with them
        if length(maxi2)>=1
            if length(maxi2)~=1
                %maximaxi2=find(1-abs(prodscal2(j,maxi2))==min(1-abs(prodscal2(j,maxi2))));
                maximaxi2=find(1./(abs(1-abs(prodscal2(j,maxi2))))==max(1./abs(1-abs(prodscal2(j,maxi2)))));
                maxi2=maxi2(maximaxi2);
            end
            width2(j)=sqrt((xb2(mindist2(j)-(SL2+1)+maxi2)-xfilt(j)).^2+(yb2(mindist2(j)-(SL2+1)+maxi2)-yfilt(j)).^2);
            maxival2(j)=maxi2;
            coord2(j,:)=[xb2(mindist2(j)-(SL2+1)+maxi2) yb2(mindist2(j)-(SL2+1)+maxi2)];
        else
            %maxi2=find(abs(prodscal2(j,:))==max(abs(prodscal2(j,:))));
            maxi2=find(1./(abs(1-abs(prodscal2(j,:))))==max(1./abs(1-abs(prodscal2(j,:)))));
            cosa=abs(prodscal2(j,maxi2));
            if length(maxi2)==1
                if (maxi2~=1).*(maxi2~=SwipeLength)
                    if projscal2(j,maxi2-1)*projscal2(j,maxi2)<0
                        cosb=abs(prodscal2(j,maxi2-1));
                        [width2(j),coord2(j,1),coord2(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb2(mindist2(j)-(SL2+1)+maxi2),yb2(mindist2(j)-(SL2+1)+maxi2),xb2(mindist2(j)-(SL2+1)+maxi2-1),yb2(mindist2(j)-(SL2+1)+maxi2-1),xdir1(j),ydir1(j));
                    elseif projscal2(j,maxi2+1)*projscal2(j,maxi2)<0
                        cosb=abs(prodscal2(j,maxi2+1));
                        [width2(j),coord2(j,1),coord2(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb2(mindist2(j)-(SL2+1)+maxi2),yb2(mindist2(j)-(SL2+1)+maxi2),xb2(mindist2(j)-(SL2+1)+maxi2+1),yb2(mindist2(j)-(SL2+1)+maxi2+1),xdir1(j),ydir1(j));
                    else %wrong position of maxi index
                        maxi2=find(projscal2(j,1:SwipeLength-1).*projscal2(j,2:SwipeLength)<0);
                        if ~isempty(maxi2)
                            if length(maxi2)>1
                                maximaxi2=find(1-abs(prodscal2(j,maxi2))==min(1-abs(prodscal2(j,maxi2))));
                                maxi2=maxi2(maximaxi2);
                            end
                            cosa=abs(prodscal2(j,maxi2));cosb=abs(prodscal2(j,maxi2+1));
                            [width2(j),coord2(j,1),coord2(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb2(mindist2(j)-(SL2+1)+maxi2),yb2(mindist2(j)-(SL2+1)+maxi2),xb2(mindist2(j)-(SL2+1)+maxi2+1),yb2(mindist2(j)-(SL2+1)+maxi2+1),xdir1(j),ydir1(j));
                        else
                            width2(j)=NaN;
                            coord2(j,:)=[NaN NaN];
                        end
                    end
                else
                    maxi2temp=maxi2;
                    maxi2=find(projscal2(j,1:SwipeLength-1).*projscal2(j,2:SwipeLength)<0);
                    if ~isempty(maxi2)%wrong position of maxi index
                        if length(maxi2)>1
                            maximaxi2=find(1-abs(prodscal2(j,maxi2))==min(1-abs(prodscal2(j,maxi2))));
                            maxi2=maxi2(maximaxi2);
                        end
                        cosa=abs(prodscal2(j,maxi2));cosb=abs(prodscal2(j,maxi2+1));
                        [width2(j),coord2(j,1),coord2(j,2)]=InterpTriangle(xfilt(j),yfilt(j),xb2(mindist2(j)-(SL2+1)+maxi2),yb2(mindist2(j)-(SL2+1)+maxi2),xb2(mindist2(j)-(SL2+1)+maxi2+1),yb2(mindist2(j)-(SL2+1)+maxi2+1),xdir1(j),ydir1(j));
                        %                 elseif prodscal2(j,:)<cosd(80)%distance accepted if angle more than 80degrees
                        %                     maxi2=maxi2temp;
                        %                     width2(j)=sqrt((xb2(mindist2(j)-(SL2+1)+maxi2)-xfilt(j)).^2+(yb2(mindist2(j)-(SL2+1)+maxi2)-yfilt(j)).^2);
                        %                     coord2(j,:)=[xb2(mindist2(j)-(SL2+1)+maxi2) yb2(mindist2(j)-(SL2+1)+maxi2)];
                    else
                        width2(j)=NaN;
                        coord2(j,:)=[NaN NaN];
                    end
                end
                if ~isempty(maxi2), maxival2(j)=maxi2; else maxival2(j)=NaN; end
            else
                width2(j)=NaN;
                coord2(j,:)=[NaN NaN];
            end
        end
    end
    %END OF FIRST PASS
    
    %III.6 - Second pass of width determinations, for pathological cases...
    %If only one point was left as NaN in the midst of other computed points,
    %there is no real hope that changing the swipe length can get that value.
    %However, if many consecutive points were left as NaN, doubling the swipe
    %length on that whole area may help. Single NaN points may be handled in an
    %updated version of the code...
    
    %III.6.1 - Bank 1
    SL3=2*SL2;%half-swipe length is doubled
    SwipeLength3=2*SL3+1;%hence swipe length SL3=2*SL-1
    nocalc1=find(isnan(width1));%Find points whose width was left as NaN after first loop
    dnocalc1=diff(nocalc1);
    ind1=find(dnocalc1~=1);%Find indices of consecutive points whose width is NaN
    indbeg1=ind1(find(diff(ind1)~=1))+1;%index of first such point within no-value points
    indend1=ind1(find(diff(ind1)~=1)+1);%index of last such point within no-value points
    nocalcbeg1=nocalc1(indbeg1);%General index of first such point
    nocalcend1=nocalc1(indend1);%General index of last such point
    remaining1=setdiff(nocalc1,[1 length(width1)]);%First and last points do not have two neighbors..
    
    %Find lonely no-value points...
    indp1=find(diff([nocalc1;nocalc1(end)+10])~=1);
    ind1del=indp1(find(diff(indp1)~=1)+1);
    indiv1=setdiff(indp1,ind1del);
    individ1=setdiff(nocalc1(indiv1),[1 length(width1)]);%First just have one neighbor..
    
    %This whole 2 following loops are basically the same thing as III.5 except
    %we do it only on the problematic groups of points, then and problematic
    %points
    for k=1:length(indbeg1)
        beg1=nocalcbeg1(k)-1;
        end1=nocalcend1(k)+1;
        firstpoint1=nocalcbeg1(k);
        lastpoint1=nocalcend1(k);
        %Instead of the minimum distance points, this part of the code assumes
        %that the center of the swipe for each centerline point has an index
        %linearly related to the points first points with values on each side of the
        %no-value interval. For instance, let's assume that points 2 to 9 had no computed
        %value in the first pass, but points 1 and 10 had. We suppose that, on a given bank,  the
        %minimum-distance point for 1 was 10 and for 10 was 100. In this part
        %of the code, we therefore would suppose that centerpoints for the
        %swipe are 20, 30, 40, ..., 70, 80, 90 for points 2 to 9...
        mindistnew1=round(mindist1(beg1)+(mindist1(end1)-mindist1(beg1))/(end1-beg1)*((firstpoint1:lastpoint1)-beg1));
        remaining1=setdiff(remaining1,firstpoint1:lastpoint1);
        matvecnewx1=xfilt(firstpoint1:lastpoint1)*ones(1,SwipeLength3);
        matvecnewy1=yfilt(firstpoint1:lastpoint1)*ones(1,SwipeLength3);
        for j=1:length(firstpoint1:lastpoint1)
            xtop1=[zeros(1,SL3-mindistnew1(j)+1)*NaN xb1(max(mindistnew1(j)-SL3,1):min(mindistnew1(j)+SL3,length(xb1)))' zeros(1,mindistnew1(j)+SL3-length(xb1))*NaN];
            ytop1=[zeros(1,SL3-mindistnew1(j)+1)*NaN yb1(max(mindistnew1(j)-SL3,1):min(mindistnew1(j)+SL3,length(yb1)))' zeros(1,mindistnew1(j)+SL3-length(yb1))*NaN];
            matvecnewx1(j,:)=xtop1-matvecnewx1(j,:);
            matvecnewy1(j,:)=ytop1-matvecnewy1(j,:);
        end
        normfactnew1=sqrt(matvecnewx1.^2+matvecnewy1.^2);
        matvecnewx1=matvecnewx1./normfactnew1;
        matvecnewy1=matvecnewy1./normfactnew1;
        prodscalnew1=(xdir1(firstpoint1:lastpoint1)*ones(1,SwipeLength3)).*matvecnewx1+(ydir1(firstpoint1:lastpoint1)*ones(1,SwipeLength3)).*matvecnewy1;
        projscalnew1=-(ydir1(firstpoint1:lastpoint1)*ones(1,SwipeLength3)).*matvecnewx1+(xdir1(firstpoint1:lastpoint1)*ones(1,SwipeLength3)).*matvecnewy1;
        widthnew1=zeros(length(firstpoint1:lastpoint1),1)*NaN;
        maxivalnew1=zeros(length(firstpoint1:lastpoint1),1)*NaN;
        coordnew1=zeros(length(firstpoint1:lastpoint1),2)*NaN;
        for j=1:length(firstpoint1:lastpoint1)
            maxi1=find(abs(round(prodscalnew1(j,:)*10^4))==10^4);
            if length(maxi1)>=1
                if length(maxi1)~=1
                    maximaxi1=find(1./(abs(1-abs(prodscalnew1(j,maxi1))))==max(1./abs(1-abs(prodscalnew1(j,maxi1)))));
                    maxi1=maxi1(maximaxi1);
                end
                widthnew1(j)=sqrt((xb1(mindistnew1(j)-(SL3+1)+maxi1)-xfilt(firstpoint1+j-1)).^2+(yb1(mindistnew1(j)-(SL3+1)+maxi1)-yfilt(firstpoint1+j-1)).^2);
                maxivalnew1(j)=maxi1;
                coordnew1(j,:)=[xb1(mindistnew1(j)-(SL3+1)+maxi1) yb1(mindistnew1(j)-(SL3+1)+maxi1)];
            else
                maxi1=find(1./(abs(1-abs(prodscalnew1(j,:))))==max(1./abs(1-abs(prodscalnew1(j,:)))));%use function 1/|1-|x|| for points closest to -1 and 1
                cosa=abs(prodscalnew1(j,maxi1));
                if length(maxi1)==1
                    if (maxi1~=1).*(maxi1~=SwipeLength3)
                        if projscalnew1(j,maxi1-1)*projscalnew1(j,maxi1)<0
                            cosb=abs(prodscalnew1(j,maxi1-1));
                            [widthnew1(j),coordnew1(j,1),coordnew1(j,2)]=InterpTriangle(xfilt(firstpoint1+j-1),yfilt(firstpoint1+j-1),xb1(mindistnew1(j)-(SL3+1)+maxi1),yb1(mindistnew1(j)-(SL3+1)+maxi1),xb1(mindistnew1(j)-(SL3+1)+maxi1-1),yb1(mindistnew1(j)-(SL3+1)+maxi1-1),xdir1(firstpoint1+j-1),ydir1(firstpoint1+j-1));
                        elseif projscalnew1(j,maxi1+1)*projscalnew1(j,maxi1)<0
                            cosb=abs(prodscalnew1(j,maxi1+1));
                            [widthnew1(j),coordnew1(j,1),coordnew1(j,2)]=InterpTriangle(xfilt(firstpoint1+j-1),yfilt(firstpoint1+j-1),xb1(mindistnew1(j)-(SL3+1)+maxi1),yb1(mindistnew1(j)-(SL3+1)+maxi1),xb1(mindistnew1(j)-(SL3+1)+maxi1+1),yb1(mindistnew1(j)-(SL3+1)+maxi1+1),xdir1(firstpoint1+j-1),ydir1(firstpoint1+j-1));
                        else %wrong position of maxi index
                            maxi1=find(projscalnew1(j,1:SwipeLength3-1).*projscalnew1(j,2:SwipeLength3)<0);
                            if ~isempty(maxi1)
                                if length(maxi1)>1
                                    maximaxi1=find(1-abs(prodscalnew1(j,maxi1))==min(1-abs(prodscalnew1(j,maxi1))));
                                    maxi1=maxi1(maximaxi1);
                                end
                                cosa=abs(prodscalnew1(j,maxi1));cosb=abs(prodscalnew1(j,maxi1+1));
                                [widthnew1(j),coordnew1(j,1),coordnew1(j,2)]=InterpTriangle(xfilt(firstpoint1+j-1),yfilt(firstpoint1+j-1),xb1(mindistnew1(j)-(SL3+1)+maxi1),yb1(mindistnew1(j)-(SL3+1)+maxi1),xb1(mindistnew1(j)-(SL3+1)+maxi1+1),yb1(mindistnew1(j)-(SL3+1)+maxi1+1),xdir1(firstpoint1+j-1),ydir1(firstpoint1+j-1));
                            else
                                widthnew1(j)=NaN;
                                coordnew1(j,:)=[NaN NaN];
                            end
                        end
                        if ~isempty(maxi1), maxivalnew1(j)=maxi1; else maxivalnew1(j)=NaN; end
                    else
                        maxi1temp=maxi1;
                        maxi1=find(projscalnew1(j,1:SwipeLength3-1).*projscalnew1(j,2:SwipeLength3)<0);
                        if ~isempty(maxi1)%wrong position of maxi index
                            if length(maxi1)>1
                                maximaxi1=find(1-abs(prodscalnew1(j,maxi1))==min(1-abs(prodscalnew1(j,maxi1))));
                                maxi1=maxi1(maximaxi1);
                            end
                            cosa=abs(prodscalnew1(j,maxi1));cosb=abs(prodscalnew1(j,maxi1+1));
                            [widthnew1(j),coordnew1(j,1),coordnew1(j,2)]=InterpTriangle(xfilt(firstpoint1+j-1),yfilt(firstpoint1+j-1),xb1(mindistnew1(j)-(SL3+1)+maxi1),yb1(mindistnew1(j)-(SL3+1)+maxi1),xb1(mindistnew1(j)-(SL3+1)+maxi1+1),yb1(mindistnew1(j)-(SL3+1)+maxi1+1),xdir1(firstpoint1+j-1),ydir1(firstpoint1+j-1));
                        elseif prodscalnew1(j,maxi1temp)<cosd(80)%distance accepted if angle more than 80degrees
                            maxi1=maxi1temp;
                            widthnew1(j)=sqrt((xb1(mindistnew1(j)-(SL3+1)+maxi1)-xfilt(firstpoint1+j-1)).^2+(yb1(mindistnew1(j)-(SL3+1)+maxi1)-yfilt(firstpoint1+j-1)).^2);
                            coordnew1(j,:)=[xb1(mindistnew1(j)-(SL3+1)+maxi1) yb1(mindistnew1(j)-(SL3+1)+maxi1)];
                        else
                            widthnew1(j)=NaN;
                            coordnew1(j,:)=[NaN NaN];
                        end
                    end
                else
                    widthnew1(j)=NaN;
                    coordnew1(j,:)=[NaN NaN];
                end
            end
        end
        %%END MID PART
        width1(firstpoint1:lastpoint1)=widthnew1;
        maxival1(firstpoint1:lastpoint1)=maxivalnew1;
        coord1(firstpoint1:lastpoint1,1)=coordnew1(:,1);
        coord1(firstpoint1:lastpoint1,2)=coordnew1(:,2);
    end
    
    %For single problematic points now, still on bank 1
    
    for k=1:length(individ1)
        %Instead of the minimum distance points, this part of the code assumes
        %that the center of the swipe for each centerline point has an index
        %that is the average index of neighboring points (rounded up). For
        %instance, let's assume that point 300 had no computed value in the
        %first pass, but points 299 and 301 had. We suppose that, on a given
        %bank,  the minimum-distance point for 299 was 150 and for 301 was 167.
        %In this part of the code, we therefore would suppose that the
        %centerpoint for the swipe @300 is 159...
        mindistnew1=round((mindist1(individ1(k)-1)+mindist1(individ1(k)+1))/2);
        matvecnewx1=xfilt(individ1(k))*ones(1,SwipeLength3);
        matvecnewy1=yfilt(individ1(k))*ones(1,SwipeLength3);
        xtop1=[zeros(1,SL3-mindistnew1+1)*NaN xb1(max(mindistnew1-SL3,1):min(mindistnew1+SL3,length(xb1)))' zeros(1,mindistnew1+SL3-length(xb1))*NaN];
        ytop1=[zeros(1,SL3-mindistnew1+1)*NaN yb1(max(mindistnew1-SL3,1):min(mindistnew1+SL3,length(yb1)))' zeros(1,mindistnew1+SL3-length(yb1))*NaN];
        matvecnewx1=xtop1-matvecnewx1;
        matvecnewy1=ytop1-matvecnewy1;
        normfactnew1=sqrt(matvecnewx1.^2+matvecnewy1.^2);
        matvecnewx1=matvecnewx1./normfactnew1;
        matvecnewy1=matvecnewy1./normfactnew1;
        prodscalnew1=(xdir1(individ1(k))*ones(1,SwipeLength3)).*matvecnewx1+(ydir1(individ1(k))*ones(1,SwipeLength3)).*matvecnewy1;
        projscalnew1=-(ydir1(individ1(k))*ones(1,SwipeLength3)).*matvecnewx1+(xdir1(individ1(k))*ones(1,SwipeLength3)).*matvecnewy1;
        widthnew1=NaN;
        maxivalnew1=NaN;
        coordnew1=zeros(1,2)*NaN;
        maxi1=find(abs(round(prodscalnew1*10^4))==10^4);
        if length(maxi1)>=1
            if length(maxi1)~=1
                maximaxi1=find(1./(abs(1-abs(prodscalnew1(maxi1))))==max(1./abs(1-abs(prodscalnew1(maxi1)))));
                maxi1=maxi1(maximaxi1);
            end
            widthnew1=sqrt((xb1(mindistnew1-(SL3+1)+maxi1)-xfilt(individ1(k))).^2+(yb1(mindistnew1-(SL3+1)+maxi1)-yfilt(individ1(k))).^2);
            maxivalnew1=maxi1;
            coordnew1=[xb1(mindistnew1-(SL3+1)+maxi1) yb1(mindistnew1-(SL3+1)+maxi1)];
        else
            maxi1=find(1./(abs(1-abs(prodscalnew1)))==max(1./abs(1-abs(prodscalnew1))));%use function 1/|1-|x|| for points closest to -1 and 1
            cosa=abs(prodscalnew1(maxi1));
            if length(maxi1)==1
                if (maxi1~=1).*(maxi1~=SwipeLength3)
                    if projscalnew1(maxi1-1)*projscalnew1(maxi1)<0
                        cosb=abs(prodscalnew1(maxi1-1));
                        [widthnew1,coordnew1(1),coordnew1(2)]=InterpTriangle(xfilt(individ1(k)),yfilt(individ1(k)),xb1(mindistnew1-(SL3+1)+maxi1),yb1(mindistnew1-(SL3+1)+maxi1),xb1(mindistnew1-(SL3+1)+maxi1-1),yb1(mindistnew1-(SL3+1)+maxi1-1),xdir1(individ1(k)),ydir1(individ1(k)));
                    elseif projscalnew1(maxi1+1)*projscalnew1(maxi1)<0
                        cosb=abs(prodscalnew1(maxi1+1));
                        [widthnew1,coordnew1(1),coordnew1(2)]=InterpTriangle(xfilt(individ1(k)),yfilt(individ1(k)),xb1(mindistnew1-(SL3+1)+maxi1),yb1(mindistnew1-(SL3+1)+maxi1),xb1(mindistnew1-(SL3+1)+maxi1+1),yb1(mindistnew1-(SL3+1)+maxi1+1),xdir1(individ1(k)),ydir1(individ1(k)));
                    else %wrong position of maxi index
                        maxi1=find(projscalnew1(1:SwipeLength3-1).*projscalnew1(2:SwipeLength3)<0);
                        if ~isempty(maxi1)
                            if length(maxi1)>1
                                maximaxi1=find(1-abs(prodscalnew1(maxi1))==min(1-abs(prodscalnew1(maxi1))));
                                maxi1=maxi1(maximaxi1);
                            end
                            cosa=abs(prodscalnew1(maxi1));cosb=abs(prodscalnew1(maxi1+1));
                            [widthnew1,coordnew1(1),coordnew1(2)]=InterpTriangle(xfilt(individ1(k)),yfilt(individ1(k)),xb1(mindistnew1-(SL3+1)+maxi1),yb1(mindistnew1-(SL3+1)+maxi1),xb1(mindistnew1-(SL3+1)+maxi1+1),yb1(mindistnew1-(SL3+1)+maxi1+1),xdir1(individ1(k)),ydir1(individ1(k)));
                        else
                            widthnew1=NaN;
                            coordnew1=[NaN NaN];
                        end
                    end
                    if ~isempty(maxi1), maxivalnew1=maxi1; else maxivalnew1=NaN; end
                else
                    maxi1temp=maxi1;
                    maxi1=find(projscalnew1(1:SwipeLength3-1).*projscalnew1(2:SwipeLength3)<0);
                    if ~isempty(maxi1)%wrong position of maxi index
                        if length(maxi1)>1
                            maximaxi1=find(1-abs(prodscalnew1(maxi1))==min(1-abs(prodscalnew1(maxi1))));
                            maxi1=maxi1(maximaxi1);
                        end
                        cosa=abs(prodscalnew1(maxi1));cosb=abs(prodscalnew1(maxi1+1));
                        [widthnew1,coordnew1(1),coordnew1(2)]=InterpTriangle(xfilt(individ1(k)),yfilt(individ1(k)),xb1(mindistnew1-(SL3+1)+maxi1),yb1(mindistnew1-(SL3+1)+maxi1),xb1(mindistnew1-(SL3+1)+maxi1+1),yb1(mindistnew1-(SL3+1)+maxi1+1),xdir1(individ1(k)),ydir1(individ1(k)));
                    elseif prodscalnew1(maxi1temp)<cosd(80)%distance accepted if angle more than 80degrees
                        maxi1=maxi1temp;
                        widthnew1=sqrt((xb1(mindistnew1-(SL3+1)+maxi1)-xfilt(individ1(k))).^2+(yb1(mindistnew1-(SL3+1)+maxi1)-yfilt(individ1(k))).^2);
                        coordnew1=[xb1(mindistnew1-(SL3+1)+maxi1) yb1(mindistnew1-(SL3+1)+maxi1)];
                    else
                        widthnew1=NaN;
                        coordnew1=[NaN NaN];
                    end
                end
            else
                widthnew1=NaN;
                coordnew1=[NaN NaN];
            end
        end
        %%END MID PART
        width1(individ1(k))=widthnew1;
        maxival1(individ1(k))=maxivalnew1;
        coord1(individ1(k),1)=coordnew1(1);
        coord1(individ1(k),2)=coordnew1(2);
    end
    
    
    
    %III.6.2 - Bank 2 - Same comments as for bank 1...
    nocalc2=find(isnan(width2));
    dnocalc2=diff(nocalc2);
    ind2=find(dnocalc2~=1);indbeg2=ind2(find(diff(ind2)~=1))+1;indend2=ind2(find(diff(ind2)~=1)+1);
    nocalcbeg2=nocalc2(indbeg2);nocalcend2=nocalc2(indend2);
    remaining2=setdiff(nocalc2,[1 length(width2)]);
    
    %Find lonely no-value points...
    indp2=find(diff([nocalc2;nocalc2(end)+10])~=1);
    ind2del=indp2(find(diff(indp2)~=1)+1);
    indiv2=setdiff(indp2,ind2del);
    individ2=setdiff(nocalc2(indiv2),[1 length(width2)]);%First just have one neighbor..
    
    for k=1:length(indbeg2)
        beg2=nocalcbeg2(k)-1;
        end2=nocalcend2(k)+1;
        firstpoint2=nocalcbeg2(k);
        lastpoint2=nocalcend2(k);
        mindistnew2=round(mindist2(beg2)+(mindist2(end2)-mindist2(beg2))/(end2-beg2)*((firstpoint2:lastpoint2)-beg2));
        remaining2=setdiff(remaining2,firstpoint2:lastpoint2);
        matvecnewx2=xfilt(firstpoint2:lastpoint2)*ones(1,SwipeLength3);
        matvecnewy2=yfilt(firstpoint2:lastpoint2)*ones(1,SwipeLength3);
        for j=1:length(firstpoint2:lastpoint2)
            xtop2=[zeros(1,SL3-mindistnew2(j)+1)*NaN xb2(max(mindistnew2(j)-SL3,1):min(mindistnew2(j)+SL3,length(xb2)))' zeros(1,mindistnew2(j)+SL3-length(xb2))*NaN];
            ytop2=[zeros(1,SL3-mindistnew2(j)+1)*NaN yb2(max(mindistnew2(j)-SL3,1):min(mindistnew2(j)+SL3,length(yb2)))' zeros(1,mindistnew2(j)+SL3-length(yb2))*NaN];
            matvecnewx2(j,:)=xtop2-matvecnewx2(j,:);
            matvecnewy2(j,:)=ytop2-matvecnewy2(j,:);
        end
        normfactnew2=sqrt(matvecnewx2.^2+matvecnewy2.^2);
        matvecnewx2=matvecnewx2./normfactnew2;
        matvecnewy2=matvecnewy2./normfactnew2;
        prodscalnew2=(xdir1(firstpoint2:lastpoint2)*ones(1,SwipeLength3)).*matvecnewx2+(ydir1(firstpoint2:lastpoint2)*ones(1,SwipeLength3)).*matvecnewy2;
        projscalnew2=-(ydir1(firstpoint2:lastpoint2)*ones(1,SwipeLength3)).*matvecnewx2+(xdir1(firstpoint2:lastpoint2)*ones(1,SwipeLength3)).*matvecnewy2;
        widthnew2=zeros(length(firstpoint2:lastpoint2),1)*NaN;
        maxivalnew2=zeros(length(firstpoint2:lastpoint2),1)*NaN;
        coordnew2=zeros(length(firstpoint2:lastpoint2),2)*NaN;
        %%MID PART
        for j=1:length(firstpoint2:lastpoint2)
            maxi2=find(abs(round(prodscalnew2(j,:)*10^4))==10^4);
            if length(maxi2)>=1
                if length(maxi2)~=1
                    maximaxi2=find(1./(abs(1-abs(prodscalnew2(j,maxi2))))==max(1./abs(1-abs(prodscalnew2(j,maxi2)))));
                    maxi2=maxi2(maximaxi2);
                end
                widthnew2(j)=sqrt((xb2(mindistnew2(j)-(SL3+1)+maxi2)-xfilt(firstpoint2+j-1)).^2+(yb2(mindistnew2(j)-(SL3+1)+maxi2)-yfilt(firstpoint2+j-1)).^2);
                maxivalnew2(j)=maxi2;
                coordnew2(j,:)=[xb2(mindistnew2(j)-(SL3+1)+maxi2) yb2(mindistnew2(j)-(SL3+1)+maxi2)];
            else
                maxi2=find(1./(abs(1-abs(prodscalnew2(j,:))))==max(1./abs(1-abs(prodscalnew2(j,:)))));%use function 1/|1-|x|| for points closest to -1 and 1
                cosa=abs(prodscalnew2(j,maxi2));
                if length(maxi2)==1
                    if (maxi2~=1).*(maxi2~=SwipeLength3)
                        if projscalnew2(j,maxi2-1)*projscalnew2(j,maxi2)<0
                            cosb=abs(prodscalnew2(j,maxi2-1));
                            [widthnew2(j),coordnew2(j,1),coordnew2(j,2)]=InterpTriangle(xfilt(firstpoint2+j-1),yfilt(firstpoint2+j-1),xb2(mindistnew2(j)-(SL3+1)+maxi2),yb2(mindistnew2(j)-(SL3+1)+maxi2),xb2(mindistnew2(j)-(SL3+1)+maxi2-1),yb2(mindistnew2(j)-(SL3+1)+maxi2-1),xdir2(firstpoint2+j-1),ydir2(firstpoint2+j-1));
                        elseif projscalnew2(j,maxi2+1)*projscalnew2(j,maxi2)<0
                            cosb=abs(prodscalnew2(j,maxi2+1));
                            [widthnew2(j),coordnew2(j,1),coordnew2(j,2)]=InterpTriangle(xfilt(firstpoint2+j-1),yfilt(firstpoint2+j-1),xb2(mindistnew2(j)-(SL3+1)+maxi2),yb2(mindistnew2(j)-(SL3+1)+maxi2),xb2(mindistnew2(j)-(SL3+1)+maxi2+1),yb2(mindistnew2(j)-(SL3+1)+maxi2+1),xdir2(firstpoint2+j-1),ydir2(firstpoint2+j-1));
                        else %wrong position of maxi index
                            maxi2=find(projscalnew2(j,1:SwipeLength3-1).*projscalnew2(j,2:SwipeLength3)<0);
                            if ~isempty(maxi2)
                                if length(maxi2)>1
                                    maximaxi2=find(1-abs(prodscalnew2(j,maxi2))==min(1-abs(prodscalnew2(j,maxi2))));
                                    maxi2=maxi2(maximaxi2);
                                end
                                cosa=abs(prodscalnew2(j,maxi2));cosb=abs(prodscalnew2(j,maxi2+1));
                                [widthnew2(j),coordnew2(j,1),coordnew2(j,2)]=InterpTriangle(xfilt(firstpoint2+j-1),yfilt(firstpoint2+j-1),xb2(mindistnew2(j)-(SL3+1)+maxi2),yb2(mindistnew2(j)-(SL3+1)+maxi2),xb2(mindistnew2(j)-(SL3+1)+maxi2+1),yb2(mindistnew2(j)-(SL3+1)+maxi2+1),xdir2(firstpoint2+j-1),ydir2(firstpoint2+j-1));
                            else
                                widthnew2(j)=NaN;
                                coordnew2(j,:)=[NaN NaN];
                            end
                        end
                        if ~isempty(maxi2), maxivalnew2(j)=maxi2; else maxivalnew2(j)=NaN; end
                    else
                        maxi2temp=maxi2;
                        maxi2=find(projscalnew2(j,1:SwipeLength3-1).*projscalnew2(j,2:SwipeLength3)<0);
                        if ~isempty(maxi2)%wrong position of maxi index
                            if length(maxi2)>1
                                maximaxi2=find(1-abs(prodscalnew2(j,maxi2))==min(1-abs(prodscalnew2(j,maxi2))));
                                maxi2=maxi2(maximaxi2);
                            end
                            cosa=abs(prodscalnew2(j,maxi2));cosb=abs(prodscalnew2(j,maxi2+1));
                            [widthnew2(j),coordnew2(j,1),coordnew2(j,2)]=InterpTriangle(xfilt(firstpoint2+j-1),yfilt(firstpoint2+j-1),xb2(mindistnew2(j)-(SL3+1)+maxi2),yb2(mindistnew2(j)-(SL3+1)+maxi2),xb2(mindistnew2(j)-(SL3+1)+maxi2+1),yb2(mindistnew2(j)-(SL3+1)+maxi2+1),xdir2(firstpoint2+j-1),ydir2(firstpoint2+j-1));
                        elseif prodscalnew2(j,maxi2temp)<cosd(80)%distance accepted if angle more than 80degrees
                            maxi2=maxi2temp;
                            widthnew2(j)=sqrt((xb2(mindistnew2(j)-(SL3+1)+maxi2)-xfilt(firstpoint2+j-1)).^2+(yb2(mindistnew2(j)-(SL3+1)+maxi2)-yfilt(firstpoint2+j-1)).^2);
                            coordnew2(j,:)=[xb2(mindistnew2(j)-(SL3+1)+maxi2) yb2(mindistnew2(j)-(SL3+1)+maxi2)];
                        else
                            widthnew2(j)=NaN;
                            coordnew2(j,:)=[NaN NaN];
                        end
                    end
                else
                    widthnew2(j)=NaN;
                    coordnew2(j,:)=[NaN NaN];
                end
            end
        end
        %%END MID PART
        width2(firstpoint2:lastpoint2)=widthnew2;
        maxival2(firstpoint2:lastpoint2)=maxivalnew2;
        coord2(firstpoint2:lastpoint2,1)=coordnew2(:,1);
        coord2(firstpoint2:lastpoint2,2)=coordnew2(:,2);
    end
    
    for k=1:length(individ2)
        %Same as above
        mindistnew2=round((mindist2(individ2(k)-1)+mindist2(individ2(k)+1))/2);
        matvecnewx2=xfilt(individ2(k))*ones(1,SwipeLength3);
        matvecnewy2=yfilt(individ2(k))*ones(1,SwipeLength3);
        xtop2=[zeros(1,SL3-mindistnew2+1)*NaN xb2(max(mindistnew2-SL3,1):min(mindistnew2+SL3,length(xb2)))' zeros(1,mindistnew2+SL3-length(xb2))*NaN];
        ytop2=[zeros(1,SL3-mindistnew2+1)*NaN yb2(max(mindistnew2-SL3,1):min(mindistnew2+SL3,length(yb2)))' zeros(1,mindistnew2+SL3-length(yb2))*NaN];
        matvecnewx2=xtop2-matvecnewx2;
        matvecnewy2=ytop2-matvecnewy2;
        normfactnew2=sqrt(matvecnewx2.^2+matvecnewy2.^2);
        matvecnewx2=matvecnewx2./normfactnew2;
        matvecnewy2=matvecnewy2./normfactnew2;
        prodscalnew2=(xdir2(individ2(k))*ones(1,SwipeLength3)).*matvecnewx2+(ydir2(individ2(k))*ones(1,SwipeLength3)).*matvecnewy2;
        projscalnew2=-(ydir2(individ2(k))*ones(1,SwipeLength3)).*matvecnewx2+(xdir2(individ2(k))*ones(1,SwipeLength3)).*matvecnewy2;
        widthnew2=NaN;
        maxivalnew2=NaN;
        coordnew2=zeros(1,2)*NaN;
        maxi2=find(abs(round(prodscalnew2*10^4))==10^4);
        if length(maxi2)>=1
            if length(maxi2)~=1
                maximaxi2=find(1./(abs(1-abs(prodscalnew2(maxi2))))==max(1./abs(1-abs(prodscalnew2(maxi2)))));
                maxi2=maxi2(maximaxi2);
            end
            widthnew2=sqrt((xb2(mindistnew2-(SL3+1)+maxi2)-xfilt(individ2(k))).^2+(yb2(mindistnew2-(SL3+1)+maxi2)-yfilt(individ2(k))).^2);
            maxivalnew2=maxi2;
            coordnew2=[xb2(mindistnew2-(SL3+1)+maxi2) yb2(mindistnew2-(SL3+1)+maxi2)];
        else
            maxi2=find(1./(abs(1-abs(prodscalnew2)))==max(1./abs(1-abs(prodscalnew2))));%use function 1/|1-|x|| for points closest to -1 and 1
            cosa=abs(prodscalnew2(maxi2));
            if length(maxi2)==1
                if (maxi2~=1).*(maxi2~=SwipeLength3)
                    if projscalnew2(maxi2-1)*projscalnew2(maxi2)<0
                        cosb=abs(prodscalnew2(maxi2-1));
                        [widthnew2,coordnew2(1),coordnew2(2)]=InterpTriangle(xfilt(individ2(k)),yfilt(individ2(k)),xb2(mindistnew2-(SL3+1)+maxi2),yb2(mindistnew2-(SL3+1)+maxi2),xb2(mindistnew2-(SL3+1)+maxi2-1),yb2(mindistnew2-(SL3+1)+maxi2-1),xdir2(individ2(k)),ydir2(individ2(k)));
                    elseif projscalnew2(maxi2+1)*projscalnew2(maxi2)<0
                        cosb=abs(prodscalnew2(maxi2+1));
                        [widthnew2,coordnew2(1),coordnew2(2)]=InterpTriangle(xfilt(individ2(k)),yfilt(individ2(k)),xb2(mindistnew2-(SL3+1)+maxi2),yb2(mindistnew2-(SL3+1)+maxi2),xb2(mindistnew2-(SL3+1)+maxi2+1),yb2(mindistnew2-(SL3+1)+maxi2+1),xdir2(individ2(k)),ydir2(individ2(k)));
                    else %wrong position of maxi index
                        maxi2=find(projscalnew2(1:SwipeLength3-1).*projscalnew2(2:SwipeLength3)<0);
                        if ~isempty(maxi2)
                            if length(maxi2)>1
                                maximaxi2=find(1-abs(prodscalnew2(maxi2))==min(1-abs(prodscalnew2(maxi2))));
                                maxi2=maxi2(maximaxi2);
                            end
                            cosa=abs(prodscalnew2(maxi2));cosb=abs(prodscalnew2(maxi2+1));
                            [widthnew2,coordnew2(1),coordnew2(2)]=InterpTriangle(xfilt(individ2(k)),yfilt(individ2(k)),xb2(mindistnew2-(SL3+1)+maxi2),yb2(mindistnew2-(SL3+1)+maxi2),xb2(mindistnew2-(SL3+1)+maxi2+1),yb2(mindistnew2-(SL3+1)+maxi2+1),xdir2(individ2(k)),ydir2(individ2(k)));
                        else
                            widthnew2=NaN;
                            coordnew2=[NaN NaN];
                        end
                    end
                    if ~isempty(maxi2), maxivalnew2=maxi2; else maxivalnew2=NaN; end
                else
                    maxi2temp=maxi2;
                    maxi2=find(projscalnew2(1:SwipeLength3-1).*projscalnew2(2:SwipeLength3)<0);
                    if ~isempty(maxi2)%wrong position of maxi index
                        if length(maxi2)>1
                            maximaxi2=find(1-abs(prodscalnew2(maxi2))==min(1-abs(prodscalnew2(maxi2))));
                            maxi2=maxi2(maximaxi2);
                        end
                        cosa=abs(prodscalnew2(maxi2));cosb=abs(prodscalnew2(maxi2+1));
                        [widthnew2,coordnew2(1),coordnew2(2)]=InterpTriangle(xfilt(individ2(k)),yfilt(individ2(k)),xb2(mindistnew2-(SL3+1)+maxi2),yb2(mindistnew2-(SL3+1)+maxi2),xb2(mindistnew2-(SL3+1)+maxi2+1),yb2(mindistnew2-(SL3+1)+maxi2+1),xdir2(individ2(k)),ydir2(individ2(k)));
                    elseif prodscalnew2(maxi2temp)<cosd(80)%distance accepted if angle more than 80degrees
                        maxi2=maxi2temp;
                        widthnew2=sqrt((xb2(mindistnew2-(SL3+1)+maxi2)-xfilt(individ2(k))).^2+(yb2(mindistnew2-(SL3+1)+maxi2)-yfilt(individ2(k))).^2);
                        coordnew2=[xb2(mindistnew2-(SL3+1)+maxi2) yb2(mindistnew2-(SL3+1)+maxi2)];
                    else
                        widthnew2=NaN;
                        coordnew2=[NaN NaN];
                    end
                end
            else
                widthnew2=NaN;
                coordnew2=[NaN NaN];
            end
        end
        %%END MID PART
        width2(individ2(k))=widthnew2;
        maxival2(individ2(k))=maxivalnew2;
        coord2(individ2(k),1)=coordnew2(1);
        coord2(individ2(k),2)=coordnew2(2);
    end
    
    
    %Now we are after two-and-a-half passes...
    
    %IV. Accuracy "estimates"
    
    %Computes coordinates of CWBP=(centerline-WBP) vectors, WBP being the bank
    %points used for width computations.
    vecx1bis=coord1(:,1)-xfilt;vecy1bis=coord1(:,2)-yfilt;
    vecx2bis=coord2(:,1)-xfilt;vecy2bis=coord2(:,2)-yfilt;
    
    normfact1bis=sqrt(vecx1bis.^2+vecy1bis.^2);
    normfact2bis=sqrt(vecx2bis.^2+vecy2bis.^2);
    
    %Coordinates of normalized CWBP vectors
    vecx1bis=vecx1bis./normfact1bis;
    vecy1bis=vecy1bis./normfact1bis;
    vecx2bis=vecx2bis./normfact2bis;
    vecy2bis=vecy2bis./normfact2bis;
    
    %Dot products between CWBP and centerline-normal vectors. Should all be close to
    %1 if everything went well
    prodscal1bis=xdir1.*vecx1bis+ydir1.*vecy1bis;
    prodscal2bis=xdir1.*vecx2bis+ydir1.*vecy2bis;
    
    %Computes full widths: ChanWidth as sum of half-widths, ChanWidth2 as full
    %width between WBP, bypassing centerline. Both should be equal in a perfect
    %world
    ChanWidth=width1+width2;
    ChanWidth2=sqrt((coord1(:,1)-coord2(:,1)).^2+(coord1(:,2)-coord2(:,2)).^2);
    
    %Accuracy estimates
    %Bank 1
    Accuracy1=sum(abs(1-abs(prodscal1bis(~isnan(prodscal1bis)))))/length(prodscal1bis(~isnan(prodscal1bis)));
    %Bank 2
    Accuracy2=sum(abs(1-abs(prodscal2bis(~isnan(prodscal2bis)))))/length(prodscal2bis(~isnan(prodscal2bis)));
    %Comparison of ChanWidth and ChanWidth2
    Accuracy3=length(find(round(ChanWidth./ChanWidth2*100)~=100))/length(ChanWidth)*100;
    
    display(['Error of centerline-normal match: ' num2str(Accuracy1*100) '% for bank 1, ' num2str(Accuracy2*100) '% for bank 2.']);
    display(['Percentage of other problematic width values: ' num2str(Accuracy3) '%. Plot ChanWidth vs ChanWidth2 to see where unreliable values are.']);
    
    %Configures outputs
    
    %Number of optional output arguments
    nout = max(nargout,1)-1;
    
    if nout>0
        varargout(1)={ChanWidth2};
    end
    if nout>1
        varargout(2)={R};
    end
    if nout>2
        varargout(3)={coord1};
    end
    if nout>3
        varargout(4)={coord2};
    end
    if nout>4
        varargout(5)={width1};
    end
    if nout>5
        varargout(6)={width2};
    end
    if nout>6
        varargout(7)={Ox};
    end
    if nout>7
        varargout(8)={Oy};
    end
    if nout>8
        varargout(9)={xdir1};
    end
    if nout>9
        varargout(10)={ydir1};
    end
    if nout>10
        coordfilt=[xfilt yfilt];
        varargout(11)={coordfilt};
    end
    if nout>11
        bankfilt1=[xb1 yb1];
        varargout(12)={bankfilt1};
    end
    if nout>12
        bankfilt2=[xb2 yb2];
        varargout(13)={bankfilt2};
    end
    if nout>13
        varargout(14)={mindist1};
    end
    if nout>14
        varargout(15)={mindist2};
    end
    if nout>15
        varargout(16)={maxival1};
    end
    if nout>16
        varargout(17)={maxival2};
    end
    endtime=toc(initime);
end

hours=floor(endtime/3600);minutes=floor((endtime-3600*hours)/60);seconds=endtime-60*minutes-3600*hours;
endtimeloop=endtime/length(x);
hoursloop=floor(endtimeloop/3600);minutesloop=floor((endtimeloop-3600*hoursloop)/60);secondsloop=endtimeloop-60*minutesloop-3600*hoursloop;
display(['Elapsed time is ' num2str(hours) 'h ' num2str(minutes) 'min ' num2str(seconds) 's, or ' num2str(hoursloop) 'h ' num2str(minutesloop) 'min ' num2str(secondsloop) 's per centerline point']);


%This is how the code may be run with all inputs and outputs... Copy-paste
%and modify to your needs!
%
%[ChanWidth,ChanWidth2,R,coord1,coord2,width1,width2,Ox,Oy,xdir,ydir,coordfilt,bankfilt1,bankfilt2,mindist1,mindist2,maxival1,maxival2]=Width(x,y,xbi1,ybi1,xbi2,ybi2,windowsize,theta,useold,SwipeLength,filteringoff);
%
%
%This is how the code may be run in "curvature-only mode" (no width
%computation, just filtering and curvature)... Copy-paste and modify to
%your needs!
%
%[R,Ox,Oy,coordfilt]=Width(x,y,NaN,NaN,NaN,NaN,windowsize,theta,useold,NaN,filteringoff);